
#ifndef __VTKBASEINSITUPIPELINE_HPP
#define __VTKBASEINSITUPIPELINE_HPP 1

#include <map>
#include <sstream>
#include <string>

#include <vtkObjectFactory.h>
#include <vtkMultiBlockDataSet.h>
#include <vtkFieldData.h>
#include <vtkCharArray.h>
#include <vtkIntArray.h>

#include <vtkCPPipeline.h>
#include <vtkCPDataDescription.h>
#include <vtkCPInputDataDescription.h>
#include <vtkLiveInsituLink.h>

#include <vtkSMPluginManager.h>
#include <vtkSMProxy.h>
#include <vtkSMProxyManager.h>
#include <vtkSMSessionProxyManager.h>
#include <vtkSMSourceProxy.h>

#include <vtkCollection.h>
#include <vtkPVPluginsInformation.h>
#include <vtkPVTrivialProducer.h>

#include <vtkSteeringPlugin.h>
#include "pluginconfig.h"

class vtkBaseInsituPipeline : public vtkCPPipeline
{
    public:
        static vtkBaseInsituPipeline* New();
        vtkTypeMacro(vtkBaseInsituPipeline,vtkCPPipeline);
        virtual void PrintSelf(ostream& os, vtkIndent indent);
        virtual void Initialize(bool doSteering = false);

        virtual int RequestDataDescription(vtkCPDataDescription* dataDescription);
        virtual int CoProcess(vtkCPDataDescription* dataDescription);
    protected:
        vtkBaseInsituPipeline();
        virtual ~vtkBaseInsituPipeline();
    private:
        void CreatePipeline(vtkCPDataDescription* dataDescription, std::string inname);
        void UpdateProducers(vtkCPDataDescription* dataDescription);
        void DoLiveVisualization(vtkCPDataDescription* dataDescription);

        vtkBaseInsituPipeline(const vtkBaseInsituPipeline&); // Not implemented
        void operator=(const vtkBaseInsituPipeline&); // Not implemented
        int OutputFrequency;
        std::string FileName;

        bool pipelineCreated;

        vtkSMSession * M_as;
        vtkSMProxyManager* M_pxm;
        vtkSMSessionProxyManager* M_spxm;
        vtkSMPluginManager* M_spm;

        std::map<std::string, vtkSmartPointer<vtkSMSourceProxy> > M_producerMap;

        vtkSmartPointer<vtkLiveInsituLink> M_insituLink;
        vtkSmartPointer<vtkSMProxy> M_proxy;

        bool M_steeringEnabled;
};

#endif
